from tuprolog import Info
from tuprolog.core import *
from tuprolog.solve import *
from tuprolog.theory.parsing import *

parser = ClausesParser.getWithDefaultOperators()

theory = parser.parseTheory(
    """
    male(james1). male(charles1). male(charles2). male(james2). male(george1).

    female(catherine). female(elizabeth). female(sophia).

    parent(charles1, james1). parent(elizabeth, james1). parent(charles2, charles1). parent(catherine, charles1).
    parent(james2, charles1). parent(sophia, elizabeth). parent(george1, sophia).

    grandparent(X, Y) :- parent(X, Z), parent(Z, Y).
    """
)

solver = Solver.getClassic().solverWithDefaultBuiltins(theory)

query = Struct.of("grandparent", Var.of("X"), Var.of("Y"))

solutions = solver.solve(query)
print(solutions)
for solution in solutions:
    print(solution)

variables = query.getVariables()
print(variables)
for variable in variables:
    print(variable)



import logging
import jpype
import jpype.imports

from it.unibo.tuprolog.theory.parsing import ClausesParser
from it.unibo.tuprolog.theory.parsing import ClausesReader

from tuprolog.core import Clause
from tuprolog.theory import Theory
from tuprolog.jvmutils import open_file, InputStream, ensure_input_steam

from tuprolog.core.operators import Operator, OperatorSet, DEFAULT_OPERATORS, EMPTY_OPERATORS

from collections.abc import Iterable
from typing import Union


logging.debug("Loaded JVM classes from it.unibo.tuprolog.theory.parsing.*")


def clauses_parser(with_default_operators: bool=True, operators: OperatorSet=None) -> ClausesParser:
    if operators is None:
        if with_default_operators:
            return ClausesParser.getWithDefaultOperators()
        else:
            return ClausesParser.getWithNoOperator()
    else:
        if with_default_operators:
            return ClausesParser.withOperators(DEFAULT_OPERATORS.plus(operators))
        else:
            return ClausesParser.withOperators(operators)


def clauses_reader(with_default_operators: bool=True, operators: OperatorSet=None) -> ClausesReader:
    if operators is None:
        if with_default_operators:
            return ClausesReader.getWithDefaultOperators()
        else:
            return ClausesReader.getWithNoOperator()
    else:
        if with_default_operators:
            return ClausesReader.withOperators(DEFAULT_OPERATORS.plus(operators))
        else:
            return ClausesReader.withOperators(operators)


DEFAULT_CLAUSES_PARSER = clauses_parser()


DEFAULT_CLAUSES_READER = clauses_reader()


def parse_theory(string: str, operators: OperatorSet=None) -> Theory:
    if operators is None:
        return DEFAULT_CLAUSES_PARSER.parseTheory(string)
    else:
        return DEFAULT_CLAUSES_PARSER.parseTheory(string, operators)


def parse_clauses(string: str, operators: OperatorSet=None, lazy: bool=True) -> Iterable[Clause]:
    if lazy:
        if operators is None:
            return DEFAULT_CLAUSES_PARSER.parseClausesLazily(string)
        else:
            return DEFAULT_CLAUSES_PARSER.parseClausesLazily(string, operators)
    else:
        if operators is None:
            return DEFAULT_CLAUSES_PARSER.parseClauses(string)
        else:
            return DEFAULT_CLAUSES_PARSER.parseClauses(string, operators)


def parse_theory(input: Union[InputStream, str], operators: OperatorSet=None) -> Theory:
    input = ensure_input_steam(input)
    if operators is None:
        return DEFAULT_CLAUSES_READER.readTheory(input)
    else:
        return DEFAULT_CLAUSES_READER.readTheory(input, operators)


def parse_clauses(input: Union[InputStream, str], operators: OperatorSet=None, lazy: bool=True) -> Iterable[Clause]:
    input = ensure_input_steam(input)
    if lazy:
        if operators is None:
            return DEFAULT_CLAUSES_READER.readClausesLazily(input)
        else:
            return DEFAULT_CLAUSES_READER.readClausesLazily(input, operators)
    else:
        if operators is None:
            return DEFAULT_CLAUSES_READER.readClauses(input)
        else:
            return DEFAULT_CLAUSES_READER.readClauses(input, operators)



import logging

logging.basicConfig(level=logging.DEBUG)

import jpype
import jpype.imports

from .libs import CLASSPATH

jars = [str(j.resolve()) for j in CLASSPATH.glob('*.jar')]

jpype.startJVM(classpath = jars)

JVM_VERSION = '.'.join(map(str, jpype.getJVMVersion()))

logging.debug("Started JVM v" + JVM_VERSION + " with classpath: " + str(jars))

from it.unibo.tuprolog import Info

logging.debug("Using 2P-Kt v" + str(Info.VERSION))



import logging
import jpype
import jpype.imports

from it.unibo.tuprolog.unify import Unificator

from tuprolog.core import Term
from tuprolog.core import Substitution
from tuprolog.pyutils import iterable_or_varargs
from tuprolog.jvmutils import jiterable, jmap

logging.debug("Loaded JVM classes from it.unibo.tuprolog.unify.*")


def strict_unificator(context: Substitution=None) -> Unificator:
    if context is None:
        return Unificator.strict()
    else:
        return Unificator.strict(context)


def naive_unificator(context: Substitution=None) -> Unificator:
    if context is None:
        return Unificator.naive()
    else:
        return Unificator.naive(context)


def cached_unificator(unificator: Unificator, cache_size: int=Unificator.DEFAULT_CACHE_CAPACITY) -> Unificator:
    return Unificator.cached(unificator, cache_size)


def unificator(context: Substitution=None) -> Unificator:
    return cached_unificator(strict_unificator(context))


DEFAULT_UNIFICATOR = Unificator.getDefault()


def mgu(x: Term, y: Term, occur_check: bool = True) -> Substitution:
    return DEFAULT_UNIFICATOR.mgu(x, y, occur_check)


def unify(x: Term, y: Term, occur_check: bool = True) -> Term:
    return DEFAULT_UNIFICATOR.unify(x, y, occur_check)


def match(x: Term, y: Term, occur_check: bool = True) -> bool:
    return DEFAULT_UNIFICATOR.match(x, y, occur_check)



import logging
import jpype
import jpype.imports

from it.unibo.tuprolog.theory import MutableTheory
from it.unibo.tuprolog.theory import Theory
from it.unibo.tuprolog.theory import RetractResult

from typing import Iterable, Union
from tuprolog.core import Clause
from tuprolog.pyutils import iterable_or_varargs
from tuprolog.jvmutils import jiterable, jmap


logging.debug("Loaded JVM classes from it.unibo.tuprolog.theory.*")


def theory(*clauses: Union[Clause, Iterable[Clause]]) -> Theory:
    return iterable_or_varargs(terms, lambda ts: Theory.of(jiterable(ts)))


def mutable_theory(*clauses: Union[Clause, Iterable[Clause]]) -> MutableTheory:
    return iterable_or_varargs(terms, lambda ts: MutableTheory.of(jiterable(ts)))